#include <QApplication>
#include <QDesktopWidget>
#include <QFileSystemModel>
#include <QFileIconProvider>
#include <QMessageBox>
#include <QTreeView>
#include <QCommandLineParser>
#include <QCommandLineOption>
#include <QContextMenuEvent>
#include <QDebug>

class TreeView : public QTreeView {
public:
    TreeView();
protected:
    void contextMenuEvent(QContextMenuEvent *event);
};

TreeView::TreeView()
{
    setAnimated(false);
    setIndentation(20);
    setSortingEnabled(true);
    qDebug() << editTriggers();
}

void TreeView::contextMenuEvent(QContextMenuEvent *event)
{
    const QModelIndex idx = indexAt(event->pos());
    if (!idx.isValid())
        return;
    qDebug() << __FUNCTION__ << idx << idx.data() << idx.flags();
    if (QMessageBox::question(this, "Del", "Del " + idx.data().toString()) != QMessageBox::Yes)
        return;
    QFileSystemModel *fModel = qobject_cast<QFileSystemModel *>(model());
    Q_ASSERT(fModel);
    bool r = fModel->remove(idx);
    qDebug() << __FUNCTION__ << "returns" << r;
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    QCoreApplication::setApplicationVersion(QT_VERSION_STR);
    QCommandLineParser parser;
    parser.setApplicationDescription("Qt Dir View Example");
    parser.addHelpOption();
    parser.addVersionOption();
    QCommandLineOption dontUseCustomDirectoryIconsOption("c", "Set QFileIconProvider::DontUseCustomDirectoryIcons");
    parser.addOption(dontUseCustomDirectoryIconsOption);
    parser.addPositionalArgument("directory", "The directory to start in.");
    parser.process(app);    
    const QString rootPath = parser.positionalArguments().value(0, QDir::tempPath());

    QFileSystemModel model;
    model.setRootPath("");
    model.setReadOnly(false);
    if (parser.isSet(dontUseCustomDirectoryIconsOption))
        model.iconProvider()->setOptions(QFileIconProvider::DontUseCustomDirectoryIcons);
    TreeView tree;
    QString t  = QLatin1String(QT_VERSION_STR)  + QLatin1Char(' ') + app.platformName();
    qDebug() << t;
    tree.setWindowTitle(t);
    tree.setModel(&model);
    if (!rootPath.isEmpty()) {
        const QModelIndex rootIndex = model.index(QDir::cleanPath(rootPath));
        if (rootIndex.isValid())
            tree.setRootIndex(rootIndex);
    }

    const QSize availableSize = QApplication::desktop()->availableGeometry(&tree).size();
    tree.resize(availableSize / 2);
    tree.resizeColumnToContents(0);

    tree.show();

    return app.exec();
}
